@@ -6,7 +6,7 @@ from PySide.QtGui import *
 from PySide.QtMaemo5 import QMaemo5ValueButton, QMaemo5ListPickSelector
 
 # Some global variables
-GnGeoGuiVersion = "0.0.5-9"
+GnGeoGuiVersion = "0.0.6-0"
 CommentChar = '#'
 OptionChar =  ' '
 Home = os.path.expanduser("~")
@@ -17,6 +17,16 @@ P1buttons = ""
 P2buttons = ""
 P1joybuttons = ""
 P2joybuttons = ""
+P1hotbuttons0 = ""
+P1hotbuttons1 = ""
+P1hotbuttons2 = ""
+P2hotbuttons0 = ""
+P2hotbuttons1 = ""
+P2hotbuttons2 = ""
+quitbutton = ""
+resetbutton = ""
+savestatebutton = ""
+loadstatebutton = ""
 BiosPath = ""
 RomrcPath = ""
 RomPath = ""
@@ -73,7 +83,8 @@ class Ui_MainWindow(QMainWindow):
         self.options = parse_config(self,GnGeo_CfgFile)
 	AllGnGeoOpts = [ "68kclock", "autoframeskip", "biospath", "country", "effect", "fullscreen", "hwsurface", 
 	                 "interpolation", "joystick", "p1joy", "p1joydev", "p2key", "p2joy", "p2joydev", "p2key", "raster", 
-	                 "romrcdir", "samplerate", "scale", "showfps", "sleepidle", "sound", "system", "z80clock" ]
+	                 "romrcdir", "samplerate", "scale", "showfps", "sleepidle", "sound", "system", "z80clock", "quit_key",
+	                 "reset_key", "load_state_key","save_state_key","p1hotkey0","p1hotkey1","p2hotkey0", "p2hotkey1"]
 	for GnGeoOpt in AllGnGeoOpts:
 		try:
 			 self.options[GnGeoOpt]
@@ -268,7 +279,7 @@ class ConfigWindow(QMainWindow):
 	wg = QWidget()
 	area.setWidget(wg)
 	self.setCentralWidget(area)
-	wg.resize(790,640);
+	wg.resize(750,660);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -460,13 +471,20 @@ class ConfigWindow(QMainWindow):
 	QObject.connect(self.joyconfButton
 		, SIGNAL("clicked()")
 		, self.on_defjoy_clicked)
+	# hotkey config
+        self.hotkeyButton = QPushButton("hotkey def",self)
+        self.hotkeyButton.setFont(font)
+        grid.addWidget(self.hotkeyButton,9,1)
+	QObject.connect(self.hotkeyButton
+		, SIGNAL("clicked()")
+		, self.on_hotkeys_clicked)
 	# Save button
         self.saveButton = QPushButton("SAVE",self)
         self.saveButton.setStyleSheet(
 	"QPushButton { color: red; font:bold; }"
       		) 
         self.saveButton.setFont(font)
-        grid.addWidget(self.saveButton,9,1,1,3)
+        grid.addWidget(self.saveButton,10,1,1,3)
 	QObject.connect(self.saveButton
 		, SIGNAL("clicked()")
 		, self.on_save_clicked)
@@ -517,34 +535,72 @@ class ConfigWindow(QMainWindow):
 		self.options["romrcdir"] = RomrcPath
 	if RomPath != "":
 		self.options["rompath"] = RomPath
+	if quitbutton != "":
+		self.options["quit_key"] = quitbutton
+	if resetbutton != "":
+		self.options["reset_key"] = resetbutton
+	if savestatebutton != "":
+		self.options["save_state_key"] = savestatebutton
+	if loadstatebutton != "":
+		self.options["load_state_key"] = loadstatebutton
 	if P1buttons != "":
 		tot_sleutel=""
 		for sleutel in P1buttons:
-			tot_sleutel = tot_sleutel + sleutel + ","
+			tot_sleutel = tot_sleutel + str(sleutel) + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p1key"] = tot_sleutel
 
 	if P2buttons != "":
 		tot_sleutel=""
 		for sleutel in P2buttons:
-			tot_sleutel = tot_sleutel + sleutel + ","
+			tot_sleutel = tot_sleutel + str(sleutel) + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p2key"] = tot_sleutel
 
 	if P1joybuttons != "":
 		tot_sleutel=""
 		for sleutel in P1joybuttons:
-			tot_sleutel = tot_sleutel + sleutel + ","
+			tot_sleutel = tot_sleutel + str(sleutel) + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p1joy"] = tot_sleutel
 
 	if P2joybuttons != "":
 		tot_sleutel=""
 		for sleutel in P2joybuttons:
-			tot_sleutel = tot_sleutel + sleutel + ","
+			tot_sleutel = tot_sleutel + str(sleutel) + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p2joy"] = tot_sleutel
 
+	if P1hotbuttons0 != "":
+		tot_sleutel=""
+		for sleutel in P1hotbuttons0:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p1hotkey0"] = tot_sleutel
+	if P1hotbuttons1 != "":
+		tot_sleutel=""
+		for sleutel in P1hotbuttons1:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p1hotkey1"] = tot_sleutel
+
+	if P2hotbuttons0 != "":
+		tot_sleutel=""
+		for sleutel in P2hotbuttons0:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p2hotkey0"] = tot_sleutel
+	if P2hotbuttons1 != "":
+		tot_sleutel=""
+		for sleutel in P2hotbuttons1:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p2hotkey1"] = tot_sleutel
+
 	if (ExecPath != "") or ((doExit == "true") and (self.gui_opts["exit_after_launch"] == "false")) or ((doExit == "false") and (self.gui_opts["exit_after_launch"] == "true")):
 		# Change in gngeo-gui config, we need to save it
 		try:
@@ -592,6 +648,10 @@ class ConfigWindow(QMainWindow):
         widget = keyWindow(self)
         widget.show()
 
+    def on_hotkeys_clicked(self):
+        widget = hotkeyWindow(self)
+        widget.show()
+
     def on_paths_clicked(self):
         widget = pathWindow(self)
         widget.show()
@@ -604,14 +664,30 @@ class keyWindow(QDialog):
     def __init__(self, parent=None):
         super(keyWindow, self).__init__(parent)
         self.setWindowTitle("GnGeo-gui button config")
-	# order : A,B,C,D,START,COIN,UP,DOWN,LEFT,RIGHT
+	# order : A,B,C,D,START,COIN,UP,DOWN,LEFT,RIGHT[,HOTKEY0,HOTKEY1]
+       	self.options = parse_config(self,GnGeo_CfgFile)
+	
+	try:
+		self.P1Resetkey=self.options["reset_key"]
+	except:
+		self.P1Resetkey=114 # 'r'
+	try:
+		self.P1Exitkey=self.options["quit_key"]
+	except:
+		self.P1Exitkey=113 # 'q'
+	try:
+		self.P1Savestatekey=self.options["save_state_key"]
+	except:
+		self.P1Savestatekey=115 # 's'
+	try:
+		self.P1Loadstatekey=self.options["load_state_key"]
+	except:
+		self.P1Loadstatekey=108 # 'l'
 	if P1buttons == "":
-        	self.options = parse_config(self,GnGeo_CfgFile)
 		self.P1Keys=self.options["p1key"].split(",")
 	else:
 		self.P1Keys = P1buttons
 	if P2buttons == "":
-        	self.options = parse_config(self,GnGeo_CfgFile)
 		self.P2Keys=self.options["p2key"].split(",")
 	else:
 		self.P2Keys = P2buttons
@@ -626,6 +702,13 @@ class keyWindow(QDialog):
 	P1Downkey=self.P1Keys[7]
 	P1Leftkey=self.P1Keys[8]
 	P1Rightkey=self.P1Keys[9]
+	# if hotkeys are not defined:
+	if len(self.P1Keys) < 11:
+		self.P1Keys.append(-1)
+	P1hotkey0=self.P1Keys[10]
+	if len(self.P1Keys) < 12:
+		self.P1Keys.append(-1)
+	P1hotkey1=self.P1Keys[11]
 	# P2 keys
 	P2Akey=self.P2Keys[0]
 	P2Bkey=self.P2Keys[1]
@@ -637,13 +720,19 @@ class keyWindow(QDialog):
 	P2Downkey=self.P2Keys[7]
 	P2Leftkey=self.P2Keys[8]
 	P2Rightkey=self.P2Keys[9]
+	if len(self.P2Keys) < 11:
+		self.P2Keys.append(-1)
+	P2hotkey0=self.P2Keys[10]
+	if len(self.P2Keys) < 12:
+		self.P2Keys.append(-1)
+	P2hotkey1=self.P2Keys[11]
 	# Layout
 	area = QScrollArea(self)
 	lay = QVBoxLayout(self)
 	lay.addWidget(area)
 	wg = QWidget()
 	area.setWidget(wg)
-	wg.resize(750,900);
+	wg.resize(750,1100);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -658,7 +747,7 @@ class keyWindow(QDialog):
 	grid.addWidget(QLabel("<center>[pick]</center>"), 1, 4)
 	# Add button list
         cntr=2
-	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT']:
+	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1','RESET','EXIT','SAVEST.','LOADST.']:
 		grid.addWidget(QLabel(pButton), cntr, 0)
 		cntr=cntr+1
 	# A-button P1
@@ -813,7 +902,7 @@ class keyWindow(QDialog):
 	grid.addWidget(self.P1RightkeySpinBox, 11, 1)
 	self.P1RightButton = MyKeyPressed("key",self)
 	grid.addWidget(self.P1RightButton, 11, 2)
-	# Right-key P1
+	# Right-key P2
 	self.P2RightkeySpinBox = QDoubleSpinBox()
 	self.P2RightkeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P2RightkeySpinBox.setValue(float(P2Rightkey))
@@ -821,8 +910,75 @@ class keyWindow(QDialog):
 	grid.addWidget(self.P2RightkeySpinBox, 11, 3)
 	self.P2RightButton = MyKeyPressed("key",self)
 	grid.addWidget(self.P2RightButton, 11, 4)
+	# Hot-key0 P1
+	self.P1Hotkey0SpinBox = QDoubleSpinBox()
+	self.P1Hotkey0SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotkey0SpinBox.setValue(float(P1hotkey0))
+	self.P1Hotkey0SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotkey0SpinBox, 12, 1)
+	self.P1Hotkey0Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P1Hotkey0Button, 12, 2)
+	# Hot-key1 P1
+	self.P1Hotkey1SpinBox = QDoubleSpinBox()
+	self.P1Hotkey1SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotkey1SpinBox.setValue(float(P1hotkey1))
+	self.P1Hotkey1SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotkey1SpinBox, 13, 1)
+	self.P1Hotkey1Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P1Hotkey1Button, 13, 2)
+	# Hot-key0 P2
+	self.P2Hotkey0SpinBox = QDoubleSpinBox()
+	self.P2Hotkey0SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotkey0SpinBox.setValue(float(P2hotkey0))
+	self.P2Hotkey0SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotkey0SpinBox, 12, 3)
+	self.P2Hotkey0Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P2Hotkey0Button, 12, 4)
+	# Hot-key1 P2
+	self.P2Hotkey1SpinBox = QDoubleSpinBox()
+	self.P2Hotkey1SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotkey1SpinBox.setValue(float(P2hotkey1))
+	self.P2Hotkey1SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotkey1SpinBox, 13, 3)
+	self.P2Hotkey1Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P2Hotkey1Button, 13, 4)
+	# Reset-key
+	self.P1ResetkeySpinBox = QDoubleSpinBox()
+	self.P1ResetkeySpinBox.setRange(minKeyValue, maxKeyValue)
+	self.P1ResetkeySpinBox.setValue(float(self.P1Resetkey))
+	self.P1ResetkeySpinBox.setDecimals(0)
+	grid.addWidget(self.P1ResetkeySpinBox, 14, 1)
+	self.P1ResetButton = MyKeyPressed("key",self)
+	grid.addWidget(self.P1ResetButton, 14, 2)
+	# Exit-key
+	self.P1ExitkeySpinBox = QDoubleSpinBox()
+	self.P1ExitkeySpinBox.setRange(minKeyValue, maxKeyValue)
+	self.P1ExitkeySpinBox.setValue(float(self.P1Exitkey))
+	self.P1ExitkeySpinBox.setDecimals(0)
+	grid.addWidget(self.P1ExitkeySpinBox, 15, 1)
+	self.P1ExitButton = MyKeyPressed("key",self)
+	grid.addWidget(self.P1ExitButton, 15, 2)
+	# Savestate-key
+	self.P1SavestatekeySpinBox = QDoubleSpinBox()
+	self.P1SavestatekeySpinBox.setRange(minKeyValue, maxKeyValue)
+	self.P1SavestatekeySpinBox.setValue(float(self.P1Savestatekey))
+	self.P1SavestatekeySpinBox.setDecimals(0)
+	grid.addWidget(self.P1SavestatekeySpinBox, 16, 1)
+	self.P1SavestateButton = MyKeyPressed("key",self)
+	grid.addWidget(self.P1SavestateButton, 16, 2)
+	# Loadstate-key
+	self.P1LoadstatekeySpinBox = QDoubleSpinBox()
+	self.P1LoadstatekeySpinBox.setRange(minKeyValue, maxKeyValue)
+	self.P1LoadstatekeySpinBox.setValue(float(self.P1Loadstatekey))
+	self.P1LoadstatekeySpinBox.setDecimals(0)
+	grid.addWidget(self.P1LoadstatekeySpinBox, 17, 1)
+	self.P1LoadstateButton = MyKeyPressed("key",self)
+	grid.addWidget(self.P1LoadstateButton, 17, 2)
 
 	self.setLayout(grid)
+        os.system('run-standalone.sh dbus-send --type=method_call --dest=org.freedesktop.Notifications  \
+                   /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint \
+                   string:"Set hotkeys to -1 to disable them"')
 	# P1
 	self.connect(self.P1AkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1BkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
@@ -834,6 +990,12 @@ class keyWindow(QDialog):
 	self.connect(self.P1DownkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1LeftkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1RightkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1ResetkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1ExitkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1Hotkey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1Hotkey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1SavestatekeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1LoadstatekeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
         self.connect(self.P1RightButton, SIGNAL("keyPressed"),self.P1updateRight)
         self.connect(self.P1LeftButton, SIGNAL("keyPressed"),self.P1updateLeft)
         self.connect(self.P1DownButton, SIGNAL("keyPressed"),self.P1updateDown)
@@ -844,6 +1006,13 @@ class keyWindow(QDialog):
         self.connect(self.P1BButton, SIGNAL("keyPressed"),self.P1updateB)
         self.connect(self.P1CButton, SIGNAL("keyPressed"),self.P1updateC)
         self.connect(self.P1DButton, SIGNAL("keyPressed"),self.P1updateD)
+        self.connect(self.P1DButton, SIGNAL("keyPressed"),self.P1updateD)
+        self.connect(self.P1Hotkey0Button, SIGNAL("keyPressed"),self.P1updateHotkey0)
+        self.connect(self.P1Hotkey1Button, SIGNAL("keyPressed"),self.P1updateHotkey1)
+        self.connect(self.P1ResetButton, SIGNAL("keyPressed"),self.P1updateReset)
+        self.connect(self.P1ExitButton, SIGNAL("keyPressed"),self.P1updateExit)
+        self.connect(self.P1SavestateButton, SIGNAL("keyPressed"),self.P1updateSavestate)
+        self.connect(self.P1LoadstateButton, SIGNAL("keyPressed"),self.P1updateLoadstate)
 	# P2
 	self.connect(self.P2AkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P2BkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
@@ -855,6 +1024,8 @@ class keyWindow(QDialog):
 	self.connect(self.P2DownkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P2LeftkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P2RightkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P2Hotkey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P2Hotkey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
         self.connect(self.P2RightButton, SIGNAL("keyPressed"),self.P2updateRight)
         self.connect(self.P2LeftButton, SIGNAL("keyPressed"),self.P2updateLeft)
         self.connect(self.P2DownButton, SIGNAL("keyPressed"),self.P2updateDown)
@@ -865,6 +1036,8 @@ class keyWindow(QDialog):
         self.connect(self.P2BButton, SIGNAL("keyPressed"),self.P2updateB)
         self.connect(self.P2CButton, SIGNAL("keyPressed"),self.P2updateC)
         self.connect(self.P2DButton, SIGNAL("keyPressed"),self.P2updateD)
+        self.connect(self.P2Hotkey0Button, SIGNAL("keyPressed"),self.P2updateHotkey0)
+        self.connect(self.P2Hotkey1Button, SIGNAL("keyPressed"),self.P2updateHotkey1)
 
 	# P1
     def P1updateRight(self):
@@ -887,6 +1060,18 @@ class keyWindow(QDialog):
 	self.P1CkeySpinBox.setValue(float(AsciiVal))
     def P1updateD(self):
 	self.P1DkeySpinBox.setValue(float(AsciiVal))
+    def P1updateReset(self):
+	self.P1ResetkeySpinBox.setValue(float(AsciiVal))
+    def P1updateExit(self):
+	self.P1ExitkeySpinBox.setValue(float(AsciiVal))
+    def P1updateSavestate(self):
+	self.P1SavestatekeySpinBox.setValue(float(AsciiVal))
+    def P1updateLoadstate(self):
+	self.P1LoadstatekeySpinBox.setValue(float(AsciiVal))
+    def P1updateHotkey0(self):
+	self.P1Hotkey0SpinBox.setValue(float(AsciiVal))
+    def P1updateHotkey1(self):
+	self.P1Hotkey1SpinBox.setValue(float(AsciiVal))
 	# P2
     def P2updateRight(self):
 	self.P2RightkeySpinBox.setValue(float(AsciiVal))
@@ -908,6 +1093,10 @@ class keyWindow(QDialog):
 	self.P2CkeySpinBox.setValue(float(AsciiVal))
     def P2updateD(self):
 	self.P2DkeySpinBox.setValue(float(AsciiVal))
+    def P2updateHotkey0(self):
+	self.P2Hotkey0SpinBox.setValue(float(AsciiVal))
+    def P2updateHotkey1(self):
+	self.P2Hotkey1SpinBox.setValue(float(AsciiVal))
 
     def change_value(self,event):
 	# P1
@@ -921,6 +1110,12 @@ class keyWindow(QDialog):
 	self.P1Keys[7] = "%.0f" % self.P1DownkeySpinBox.value()
 	self.P1Keys[8] = "%.0f" % self.P1LeftkeySpinBox.value()
 	self.P1Keys[9] = "%.0f" % self.P1RightkeySpinBox.value()
+	self.P1Keys[10] = "%.0f" % self.P1Hotkey0SpinBox.value()
+	self.P1Keys[11] = "%.0f" % self.P1Hotkey1SpinBox.value()
+	self.P1Resetkey = "%.0f" % self.P1ResetkeySpinBox.value()
+	self.P1Exitkey = "%.0f" % self.P1ExitkeySpinBox.value()
+	self.P1Savestatekey = "%.0f" % self.P1SavestatekeySpinBox.value()
+	self.P1Loadstatekey = "%.0f" % self.P1LoadstatekeySpinBox.value()
 	# P2
 	self.P2Keys[0] = "%.0f" % self.P2AkeySpinBox.value()
 	self.P2Keys[1] = "%.0f" % self.P2BkeySpinBox.value()
@@ -932,10 +1127,20 @@ class keyWindow(QDialog):
 	self.P2Keys[7] = "%.0f" % self.P2DownkeySpinBox.value()
 	self.P2Keys[8] = "%.0f" % self.P2LeftkeySpinBox.value()
 	self.P2Keys[9] = "%.0f" % self.P2RightkeySpinBox.value()
+	self.P2Keys[10] = "%.0f" % self.P2Hotkey0SpinBox.value()
+	self.P2Keys[11] = "%.0f" % self.P2Hotkey1SpinBox.value()
 	global P1buttons
 	global P2buttons
+	global quitbutton
+	global resetbutton
+	global savestatebutton
+	global loadstatebutton
 	P1buttons = self.P1Keys
 	P2buttons = self.P2Keys
+	quitbutton = self.P1Exitkey
+	resetbutton = self.P1Resetkey
+	savestatebutton = self.P1Savestatekey
+	loadstatebutton = self.P1Loadstatekey
 
 class JoykeyWindow(QDialog):
     def __init__(self, parent=None):
@@ -967,6 +1172,13 @@ class JoykeyWindow(QDialog):
 	P1Downjoykey=self.P1joyKeys[7]
 	P1Leftjoykey=self.P1joyKeys[8]
 	P1Rightjoykey=self.P1joyKeys[9]
+	# if hotkeys are not defined:
+	if len(self.P1joyKeys) < 11:
+		self.P1joyKeys.append(-1)
+	P1hotjoykey0=self.P1joyKeys[10]
+	if len(self.P1joyKeys) < 12:
+		self.P1joyKeys.append(-1)
+	P1hotjoykey1=self.P1joyKeys[11]
 	# P2 keys
 	P2Ajoykey=self.P2joyKeys[0]
 	P2Bjoykey=self.P2joyKeys[1]
@@ -978,6 +1190,13 @@ class JoykeyWindow(QDialog):
 	P2Downjoykey=self.P2joyKeys[7]
 	P2Leftjoykey=self.P2joyKeys[8]
 	P2Rightjoykey=self.P2joyKeys[9]
+	# if hotkeys are not defined:
+	if len(self.P2joyKeys) < 11:
+		self.P2joyKeys.append(-1)
+	P2hotjoykey0=self.P2joyKeys[10]
+	if len(self.P2joyKeys) < 12:
+		self.P2joyKeys.append(-1)
+	P2hotjoykey1=self.P2joyKeys[11]
 	# Layout
 	area = QScrollArea(self)
 	lay = QVBoxLayout(self)
@@ -1000,7 +1219,7 @@ class JoykeyWindow(QDialog):
 		grid.addWidget(QLabel("<center>[pick]</center>"), 1, 4)
 	# Add button list
         cntr=2
-	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT']:
+	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1']:
 		grid.addWidget(QLabel(pButton), cntr, 0)
 		cntr=cntr+1
 	# A-button P1
@@ -1117,12 +1336,36 @@ class JoykeyWindow(QDialog):
 	self.P1RightjoykeySpinBox.setValue(float(P1Rightjoykey))
 	self.P1RightjoykeySpinBox.setDecimals(0)
 	grid.addWidget(self.P1RightjoykeySpinBox, 11, 1)
-	# Right-key P1
+	# Right-key P2
 	self.P2RightjoykeySpinBox = QDoubleSpinBox()
 	self.P2RightjoykeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P2RightjoykeySpinBox.setValue(float(P2Rightjoykey))
 	self.P2RightjoykeySpinBox.setDecimals(0)
 	grid.addWidget(self.P2RightjoykeySpinBox, 11, 3)
+	# hotkey0 P1
+	self.P1Hotjoykey0SpinBox = QDoubleSpinBox()
+	self.P1Hotjoykey0SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotjoykey0SpinBox.setValue(float(P1hotjoykey0))
+	self.P1Hotjoykey0SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotjoykey0SpinBox, 12, 1)
+	# hotkey0 P2
+	self.P2Hotjoykey0SpinBox = QDoubleSpinBox()
+	self.P2Hotjoykey0SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotjoykey0SpinBox.setValue(float(P2hotjoykey0))
+	self.P2Hotjoykey0SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotjoykey0SpinBox, 12, 3)
+	# hotkey1 P1
+	self.P1Hotjoykey1SpinBox = QDoubleSpinBox()
+	self.P1Hotjoykey1SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotjoykey1SpinBox.setValue(float(P1hotjoykey1))
+	self.P1Hotjoykey1SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotjoykey1SpinBox, 13, 1)
+	# hotkey1 P2
+	self.P2Hotjoykey1SpinBox = QDoubleSpinBox()
+	self.P2Hotjoykey1SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotjoykey1SpinBox.setValue(float(P2hotjoykey1))
+	self.P2Hotjoykey1SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotjoykey1SpinBox, 13, 3)
 
 	if (joystickAttached):
 		self.P2AjoyButton = QPushButton("button",self)
@@ -1145,10 +1388,18 @@ class JoykeyWindow(QDialog):
 		self.P1LeftjoyButton = QPushButton("button",self)
 		self.P2DownjoyButton = QPushButton("button",self)
 		self.P1DownjoyButton = QPushButton("button",self)
+		self.P1Hotkey0joyButton = QPushButton("button",self)
+		self.P1Hotkey1joyButton = QPushButton("button",self)
+		self.P2Hotkey0joyButton = QPushButton("button",self)
+		self.P2Hotkey1joyButton = QPushButton("button",self)
 		grid.addWidget(self.P1AjoyButton, 2, 2)
 		grid.addWidget(self.P2AjoyButton, 2, 4)
 		grid.addWidget(self.P2RightjoyButton, 11, 4)
 		grid.addWidget(self.P1RightjoyButton, 11, 2)
+		grid.addWidget(self.P1Hotkey0joyButton, 12, 2)
+		grid.addWidget(self.P2Hotkey0joyButton, 12, 4)
+		grid.addWidget(self.P1Hotkey1joyButton, 13, 2)
+		grid.addWidget(self.P2Hotkey1joyButton, 13, 4)
 		grid.addWidget(self.P2LeftjoyButton, 10, 4)
 		grid.addWidget(self.P2DownjoyButton, 9, 4)
 		grid.addWidget(self.P1LeftjoyButton, 10, 2)
@@ -1167,6 +1418,9 @@ class JoykeyWindow(QDialog):
 		grid.addWidget(self.P2CjoyButton, 4, 4)
 
 	self.setLayout(grid)
+        os.system('run-standalone.sh dbus-send --type=method_call --dest=org.freedesktop.Notifications  \
+                   /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint \
+                   string:"Set hotkeys to -1 to disable them"')
 	# P1
 	self.connect(self.P1AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P1BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
@@ -1178,6 +1432,8 @@ class JoykeyWindow(QDialog):
 	self.connect(self.P1DownjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P1LeftjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P1RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1Hotjoykey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1Hotjoykey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	if (joystickAttached):
 		self.connect(self.P1RightjoyButton, SIGNAL("clicked()"),self.P1joyupdateRight)
 		self.connect(self.P1LeftjoyButton, SIGNAL("clicked()"),self.P1updateLeft)
@@ -1189,6 +1445,8 @@ class JoykeyWindow(QDialog):
 		self.connect(self.P1BjoyButton, SIGNAL("clicked()"),self.P1updateB)
 		self.connect(self.P1CjoyButton, SIGNAL("clicked()"),self.P1updateC)
 		self.connect(self.P1DjoyButton, SIGNAL("clicked()"),self.P1updateD)
+		self.connect(self.P1Hotkey0joyButton, SIGNAL("clicked()"),self.P1updateHotkey0)
+		self.connect(self.P1Hotkey1joyButton, SIGNAL("clicked()"),self.P1updateHotkey1)
 	# P2
 	self.connect(self.P2AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P2BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
@@ -1211,6 +1469,8 @@ class JoykeyWindow(QDialog):
 		self.connect(self.P2BjoyButton, SIGNAL("clicked()"),self.P2updateB)
 		self.connect(self.P2CjoyButton, SIGNAL("clicked()"),self.P2updateC)
 		self.connect(self.P2DjoyButton, SIGNAL("clicked()"),self.P2updateD)
+		self.connect(self.P2Hotkey0joyButton, SIGNAL("clicked()"),self.P2updateHotkey0)
+		self.connect(self.P2Hotkey1joyButton, SIGNAL("clicked()"),self.P2updateHotkey1)
 
 	# P1
     def P1joyupdateRight(self):
@@ -1243,6 +1503,13 @@ class JoykeyWindow(QDialog):
     def P1updateD(self):
 	MyJoykeyPressed()
 	self.P1DjoykeySpinBox.setValue(float(joyVal))
+    def P1updateHotkey0(self):
+	MyJoykeyPressed()
+	self.P1Hotjoykey0SpinBox.setValue(float(joyVal))
+    def P1updateHotkey1(self):
+	MyJoykeyPressed()
+	self.P1Hotjoykey1SpinBox.setValue(float(joyVal))
+
 	# P2
     def P2updateRight(self):
 	MyJoykeyPressed()
@@ -1274,6 +1541,12 @@ class JoykeyWindow(QDialog):
     def P2updateD(self):
 	MyJoykeyPressed()
 	self.P2DjoykeySpinBox.setValue(float(joyVal))
+    def P2updateHotkey0(self):
+	MyJoykeyPressed()
+	self.P2Hotjoykey0SpinBox.setValue(float(joyVal))
+    def P2updateHotkey1(self):
+	MyJoykeyPressed()
+	self.P2Hotjoykey1SpinBox.setValue(float(joyVal))
 
     def change_value_joy(self):
 	# P1
@@ -1287,6 +1560,8 @@ class JoykeyWindow(QDialog):
 	self.P1joyKeys[7] = "%.0f" % self.P1DownjoykeySpinBox.value()
 	self.P1joyKeys[8] = "%.0f" % self.P1LeftjoykeySpinBox.value()
 	self.P1joyKeys[9] = "%.0f" % self.P1RightjoykeySpinBox.value()
+	self.P1joyKeys[10] = "%.0f" % self.P1Hotjoykey0SpinBox.value()
+	self.P1joyKeys[11] = "%.0f" % self.P1Hotjoykey1SpinBox.value()
 	# P2
 	self.P2joyKeys[0] = "%.0f" % self.P2AjoykeySpinBox.value()
 	self.P2joyKeys[1] = "%.0f" % self.P2BjoykeySpinBox.value()
@@ -1298,6 +1573,8 @@ class JoykeyWindow(QDialog):
 	self.P2joyKeys[7] = "%.0f" % self.P2DownjoykeySpinBox.value()
 	self.P2joyKeys[8] = "%.0f" % self.P2LeftjoykeySpinBox.value()
 	self.P2joyKeys[9] = "%.0f" % self.P2RightjoykeySpinBox.value()
+	self.P2joyKeys[10] = "%.0f" % self.P2Hotjoykey0SpinBox.value()
+	self.P2joyKeys[11] = "%.0f" % self.P2Hotjoykey1SpinBox.value()
 	global P1joybuttons
 	global P2joybuttons
 	P1joybuttons = self.P1joyKeys
@@ -1390,6 +1667,316 @@ class MyJoykeyPressed(QPushButton):
 			break
 	pipe.close()
 
+class hotkeyWindow(QDialog):
+    def __init__(self, parent=None):
+        super(hotkeyWindow, self).__init__(parent)
+        self.setWindowTitle("GnGeo-gui hotkey config")
+       	self.options = parse_config(self,GnGeo_CfgFile)
+	area = QScrollArea(self)
+	lay = QVBoxLayout(self)
+	lay.addWidget(area)
+	wg = QWidget()
+	area.setWidget(wg)
+	wg.setMinimumSize(700,300);
+	grid = QGridLayout(wg)
+	wg.show()
+	area.show()
+	grid.setColumnStretch(1, 20)
+	# bios path
+        font = QFont()
+        font.setPointSize(24)
+	if P1hotbuttons0 == "":
+		self.P1hotKeys0=self.options["p1hotkey0"].split(",")
+	else:
+		self.P1hotKeys0 = P1hotbuttons0
+	if P1hotbuttons1 == "":
+		self.P1hotKeys1=self.options["p1hotkey1"].split(",")
+	else:
+		self.P1hotKeys1 = P1hotbuttons1
+	if P2hotbuttons0 == "":
+		self.P2hotKeys0=self.options["p2hotkey0"].split(",")
+	else:
+		self.P2hotKeys0 = P2hotbuttons0
+	if P2hotbuttons1 == "":
+		self.P2hotKeys1=self.options["p2hotkey1"].split(",")
+	else:
+		self.P2hotKeys1 = P2hotbuttons1
+	grid.addWidget(QLabel("P1 hotkey 0"), 1, 1)
+	grid.addWidget(QLabel("P1 hotkey 1"), 2, 1)
+	grid.addWidget(QLabel("P2 hotkey 0"), 4, 1)
+	grid.addWidget(QLabel("P2 hotkey 1"), 5, 1)
+	# p1hotkey0
+        self.P1A0PushButton = QPushButton("A",self)
+	self.P1A0PushButton.setCheckable(True)
+        grid.addWidget(self.P1A0PushButton,1,2)
+        self.P1B0PushButton = QPushButton("B",self)
+	self.P1B0PushButton.setCheckable(True)
+        grid.addWidget(self.P1B0PushButton,1,3)
+        self.P1C0PushButton = QPushButton("C",self)
+	self.P1C0PushButton.setCheckable(True)
+        grid.addWidget(self.P1C0PushButton,1,4)
+        self.P1D0PushButton = QPushButton("D",self)
+	self.P1D0PushButton.setCheckable(True)
+        grid.addWidget(self.P1D0PushButton,1,5)
+	# p1hotkey1
+        self.P1A1PushButton = QPushButton("A",self)
+	self.P1A1PushButton.setCheckable(True)
+        grid.addWidget(self.P1A1PushButton,2,2)
+        self.P1B1PushButton = QPushButton("B",self)
+	self.P1B1PushButton.setCheckable(True)
+        grid.addWidget(self.P1B1PushButton,2,3)
+        self.P1C1PushButton = QPushButton("C",self)
+	self.P1C1PushButton.setCheckable(True)
+        grid.addWidget(self.P1C1PushButton,2,4)
+        self.P1D1PushButton = QPushButton("D",self)
+	self.P1D1PushButton.setCheckable(True)
+        grid.addWidget(self.P1D1PushButton,2,5)
+	# p2hotkey0
+        self.P2A0PushButton = QPushButton("A",self)
+	self.P2A0PushButton.setCheckable(True)
+        grid.addWidget(self.P2A0PushButton,4,2)
+        self.P2B0PushButton = QPushButton("B",self)
+	self.P2B0PushButton.setCheckable(True)
+        grid.addWidget(self.P2B0PushButton,4,3)
+        self.P2C0PushButton = QPushButton("C",self)
+	self.P2C0PushButton.setCheckable(True)
+        grid.addWidget(self.P2C0PushButton,4,4)
+        self.P2D0PushButton = QPushButton("D",self)
+	self.P2D0PushButton.setCheckable(True)
+        grid.addWidget(self.P2D0PushButton,4,5)
+	# p2hotkey1
+        self.P2A1PushButton = QPushButton("A",self)
+	self.P2A1PushButton.setCheckable(True)
+        grid.addWidget(self.P2A1PushButton,5,2)
+        self.P2B1PushButton = QPushButton("B",self)
+	self.P2B1PushButton.setCheckable(True)
+        grid.addWidget(self.P2B1PushButton,5,3)
+        self.P2C1PushButton = QPushButton("C",self)
+	self.P2C1PushButton.setCheckable(True)
+        grid.addWidget(self.P2C1PushButton,5,4)
+        self.P2D1PushButton = QPushButton("D",self)
+	self.P2D1PushButton.setCheckable(True)
+        grid.addWidget(self.P2D1PushButton,5,5)
+
+	# A = 1, B = 2, C = 4, D = 8
+	# p1hotkey0
+	if "1" in self.P1hotKeys0:
+		self.P1A0PushButton.setChecked(True)
+	else:
+		self.P1hotKeys0.insert(0,"")
+	if "2" in self.P1hotKeys0:
+		self.P1B0PushButton.setChecked(True)
+	else:
+		self.P1hotKeys0.insert(1,"")
+	if "4" in self.P1hotKeys0:
+		self.P1C0PushButton.setChecked(True)
+	else:
+		self.P1hotKeys0.insert(2,"")
+	if "8" in self.P1hotKeys0:
+		self.P1D0PushButton.setChecked(True)
+	else:
+		self.P1hotKeys0.append("")
+	# p1hotkey1
+	if "1" in self.P1hotKeys1:
+		self.P1A1PushButton.setChecked(True)
+	else:
+		self.P1hotKeys1.insert(0,"")
+	if "2" in self.P1hotKeys1:
+		self.P1B1PushButton.setChecked(True)
+	else:
+		self.P1hotKeys1.insert(1,"")
+	if "4" in self.P1hotKeys1:
+		self.P1C1PushButton.setChecked(True)
+	else:
+		self.P1hotKeys1.insert(2,"")
+	if "8" in self.P1hotKeys1:
+		self.P1D1PushButton.setChecked(True)
+	else:
+		self.P1hotKeys1.append("")
+	# p2hotkey0
+	if "1" in self.P2hotKeys0:
+		self.P2A0PushButton.setChecked(True)
+	else:
+		self.P2hotKeys0.insert(0,"")
+	if "2" in self.P2hotKeys0:
+		self.P2B0PushButton.setChecked(True)
+	else:
+		self.P2hotKeys0.insert(1,"")
+	if "4" in self.P2hotKeys0:
+		self.P2C0PushButton.setChecked(True)
+	else:
+		self.P2hotKeys0.insert(2,"")
+	if "8" in self.P2hotKeys0:
+		self.P2D0PushButton.setChecked(True)
+	else:
+		self.P2hotKeys0.append("")
+	# p1hotkey1
+	if "1" in self.P2hotKeys1:
+		self.P2A1PushButton.setChecked(True)
+	else:
+		self.P2hotKeys1.insert(0,"")
+	if "2" in self.P2hotKeys1:
+		self.P2B1PushButton.setChecked(True)
+	else:
+		self.P2hotKeys1.insert(1,"")
+	if "4" in self.P2hotKeys1:
+		self.P2C1PushButton.setChecked(True)
+	else:
+		self.P2hotKeys1.insert(2,"")
+	if "8" in self.P2hotKeys1:
+		self.P2D1PushButton.setChecked(True)
+	else:
+		self.P2hotKeys1.append("")
+
+	# Connect to P1 hotkey buttons
+	self.connect(self.P1A0PushButton, SIGNAL("clicked()"),self.P1A0update)
+	self.connect(self.P1B0PushButton, SIGNAL("clicked()"),self.P1B0update)
+	self.connect(self.P1C0PushButton, SIGNAL("clicked()"),self.P1C0update)
+	self.connect(self.P1D0PushButton, SIGNAL("clicked()"),self.P1D0update)
+	self.connect(self.P1A1PushButton, SIGNAL("clicked()"),self.P1A1update)
+	self.connect(self.P1B1PushButton, SIGNAL("clicked()"),self.P1B1update)
+	self.connect(self.P1C1PushButton, SIGNAL("clicked()"),self.P1C1update)
+	self.connect(self.P1D1PushButton, SIGNAL("clicked()"),self.P1D1update)
+	# Connect to P1 hotkey buttons
+	self.connect(self.P2A0PushButton, SIGNAL("clicked()"),self.P2A0update)
+	self.connect(self.P2B0PushButton, SIGNAL("clicked()"),self.P2B0update)
+	self.connect(self.P2C0PushButton, SIGNAL("clicked()"),self.P2C0update)
+	self.connect(self.P2D0PushButton, SIGNAL("clicked()"),self.P2D0update)
+	self.connect(self.P2A1PushButton, SIGNAL("clicked()"),self.P2A1update)
+	self.connect(self.P2B1PushButton, SIGNAL("clicked()"),self.P2B1update)
+	self.connect(self.P2C1PushButton, SIGNAL("clicked()"),self.P2C1update)
+	self.connect(self.P2D1PushButton, SIGNAL("clicked()"),self.P2D1update)
+
+	# player 1 hotkey update
+    def P1A0update(self):
+	global P1hotbuttons0
+	if self.P1A0PushButton.isChecked():
+		self.P1hotKeys0[0] = "1"
+	else:
+		self.P1hotKeys0[0] = ""
+	P1hotbuttons0 = self.P1hotKeys0
+
+    def P1B0update(self):
+	global P1hotbuttons0
+	if self.P1B0PushButton.isChecked():
+		self.P1hotKeys0[1] = "2"
+	else:
+		self.P1hotKeys0[1] = ""
+	P1hotbuttons0 = self.P1hotKeys0
+ 
+    def P1C0update(self):
+	global P1hotbuttons0
+	if self.P1C0PushButton.isChecked():
+		self.P1hotKeys0[2] = "4"
+	else:
+		self.P1hotKeys0[2] = ""
+	P1hotbuttons0 = self.P1hotKeys0
+ 
+    def P1D0update(self):
+	global P1hotbuttons0
+	if self.P1D0PushButton.isChecked():
+		self.P1hotKeys0[3] = "8"
+	else:
+		self.P1hotKeys0[3] = ""
+	P1hotbuttons0 = self.P1hotKeys0
+
+    def P1A1update(self):
+	global P1hotbuttons1
+	if self.P1A1PushButton.isChecked():
+		self.P1hotKeys1[0] = "1"
+	else:
+		self.P1hotKeys1[0] = ""
+	P1hotbuttons1 = self.P1hotKeys1
+
+    def P1B1update(self):
+	global P1hotbuttons1
+	if self.P1B1PushButton.isChecked():
+		self.P1hotKeys1[1] = "2"
+	else:
+		self.P1hotKeys1[1] = ""
+	P1hotbuttons1 = self.P1hotKeys1
+ 
+    def P1C1update(self):
+	global P1hotbuttons1
+	if self.P1C1PushButton.isChecked():
+		self.P1hotKeys1[2] = "4"
+	else:
+		self.P1hotKeys1[2] = ""
+	P1hotbuttons1 = self.P1hotKeys1
+ 
+    def P1D1update(self):
+	global P1hotbuttons1
+	if self.P1D1PushButton.isChecked():
+		self.P1hotKeys1[3] = "8"
+	else:
+		self.P1hotKeys1[3] = ""
+	P1hotbuttons1 = self.P1hotKeys1
+ 
+	# player 2 hotkey update
+    def P2A0update(self):
+	global P2hotbuttons0
+	if self.P2A0PushButton.isChecked():
+		self.P2hotKeys0[0] = "1"
+	else:
+		self.P2hotKeys0[0] = ""
+	P2hotbuttons0 = self.P2hotKeys0
+
+    def P2B0update(self):
+	global P2hotbuttons0
+	if self.P2B0PushButton.isChecked():
+		self.P2hotKeys0[1] = "2"
+	else:
+		self.P2hotKeys0[1] = ""
+	P2hotbuttons0 = self.P2hotKeys0
+ 
+    def P2C0update(self):
+	global P2hotbuttons0
+	if self.P2C0PushButton.isChecked():
+		self.P2hotKeys0[2] = "4"
+	else:
+		self.P2hotKeys0[2] = ""
+	P2hotbuttons0 = self.P2hotKeys0
+ 
+    def P2D0update(self):
+	global P2hotbuttons0
+	if self.P2D0PushButton.isChecked():
+		self.P2hotKeys0[3] = "8"
+	else:
+		self.P2hotKeys0[3] = ""
+	P2hotbuttons0 = self.P2hotKeys0
+
+    def P2A1update(self):
+	global P2hotbuttons1
+	if self.P2A1PushButton.isChecked():
+		self.P2hotKeys1[0] = "1"
+	else:
+		self.P2hotKeys1[0] = ""
+	P2hotbuttons1 = self.P2hotKeys1
+
+    def P2B1update(self):
+	global P2hotbuttons1
+	if self.P2B1PushButton.isChecked():
+		self.P2hotKeys1[1] = "2"
+	else:
+		self.P2hotKeys1[1] = ""
+	P2hotbuttons1 = self.P2hotKeys1
+ 
+    def P2C1update(self):
+	global P2hotbuttons1
+	if self.P2C1PushButton.isChecked():
+		self.P2hotKeys1[2] = "4"
+	else:
+		self.P2hotKeys1[2] = ""
+	P2hotbuttons1 = self.P2hotKeys1
+ 
+    def P2D1update(self):
+	global P2hotbuttons1
+	if self.P2D1PushButton.isChecked():
+		self.P2hotKeys1[3] = "8"
+	else:
+		self.P2hotKeys1[3] = ""
+	P2hotbuttons1 = self.P2hotKeys1
+ 
  
 class pathWindow(QDialog):
     def __init__(self, parent=None):
@@ -1480,11 +2067,53 @@ class pathWindow(QDialog):
 	if (not os.path.isfile(RomrcPath + "/kof96.rc")) and (RomrcPath != "") :
 		QMessageBox.critical(self, "Warning",  "Cannot find " + RomrcPath + "/*.rc files")
 	self.close()
-
     def on_rom_button_clicked(self):
-	global RomPath
-	RomPath = QFileDialog.getExistingDirectory(self, 'Choose directory', '/home')
-	self.close()
+        d = QDialog(self)
+
+        vbox = QVBoxLayout()
+        #Add Layouts
+        horizontalLayout = QHBoxLayout()
+
+        # Now create your buttons.
+        homeButton = QPushButton("internal")
+        rootButton = QPushButton("SD card")
+        closeButton = QPushButton("Close")
+        # Add buttons to your layouts, they'll show up in the order given.
+        horizontalLayout.addWidget(homeButton)
+        horizontalLayout.addWidget(rootButton)
+        horizontalLayout.addWidget(closeButton)
+        # Add layouts to your main Layout
+        vbox.addLayout (horizontalLayout)
+        d.setLayout(vbox)
+        d.setWindowTitle("Choose roms location...")
+
+	# Show the window!
+        d.show()
+        QObject.connect(homeButton
+                , SIGNAL("clicked()")
+                , self.on_rom_homebutton_clicked)
+        QObject.connect(rootButton
+                , SIGNAL("clicked()")
+                , self.on_rom_rootbutton_clicked)
+        QObject.connect(closeButton
+                , SIGNAL("clicked()")
+                , d.close)
+
+    def on_rom_homebutton_clicked(self):
+        global RomPath
+        RomPath = QFileDialog.getExistingDirectory(self, 'Choose directory', '/home')
+        self.close()
+        os.system('run-standalone.sh dbus-send --type=method_call --dest=org.freedesktop.Notifications  \
+                   /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint \
+                   string:"Use SAVE button to save it!"')
+
+    def on_rom_rootbutton_clicked(self):
+        global RomPath
+        RomPath = QFileDialog.getExistingDirectory(self, 'Choose directory', '/media')
+        self.close()
+        os.system('run-standalone.sh dbus-send --type=method_call --dest=org.freedesktop.Notifications  \
+                   /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteInfoprint \
+                   string:"Use SAVE button to save it!"')
 
     def on_exec_button_clicked(self):
 	global ExecPath
